function demo_l2_TV
% TV based image deburring using TwIST:
% Setting: cameraman, blur uniform 9*9, BSNR = 40 dB
%
% This demo illustrates the computation of  
%
%     xe = arg min 0.5 ||Ax-y||^2 + tau TV(x)
%             x
% 
% with the TwIST algorithm, where A denotes the convolution operator
%
% The proximal operator, i.e., the solution of
%
%     xe = arg min 0.5 ||x-y||^2 + tau TV(x)
%             x
% is  given by the Chambolle algorithm
% 
% A. Chambolle, An algorithm for total variation minimization and
% applications, Journal of Mathematical Imaging and Vision, vol. 20,
% pp. 89-97, 2004.
%
%
% For further details about the TwIST algorithm, see the paper:
%
% J. Bioucas-Dias and M. Figueiredo, "A New TwIST: Two-Step
% Iterative Shrinkage/Thresholding Algorithms for Image 
% Restoration",  IEEE Transactions on Image processing, 2007.
% 
%
% (available at   http://www.lx.it.pt/~bioucas/publications.html)
%
%
% 
% Authors: Jose Bioucas-Dias and Mario Figueiredo, 
% Instituto Superior Tcnico, October, 2007




%%%%%%%%%%%%%%%%%%%%% Original Data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x = double(imread('cameraman.tif'));
N=length(x);
colormap(gray);

% remove mean (not necessary)
mu=mean(x(:));
x=x-mu;

% define the blur operator
% N must be even
middle=N/2+1;
% blurr matrix
B=zeros(N);
% Uniform blur
lx=4; %blur x-size
ly=4; %blurr y-size
B((middle-ly):(middle+ly),(middle-lx):(middle+lx))=1;
%circularly center
B=fftshift(B);
%normalize (not necessary)
B=B/sum(sum(B));
% convolve
y = real(ifft2(fft2(B).*fft2(x)));

% set BSNR
BSNR = 40;
Py = var(y(:));
sigma= sqrt((Py/10^(BSNR/10)));

% add noise
y=y+ sigma*randn(N);

% plot figures
figure(1); colormap gray; 
imagesc(x); axis off;
title('Original image')
figure(2); colormap gray; 
title('Noisy and blurred image')
imagesc(y); axis off;


% smoothing parameter (empirical)
tau = 1.9e-2*sigma^2/0.56^2;

% ------------  TV experiments ---------------------
K=fft2(B);
KC=conj(K);

% handle functions for TwIST
%  convolution operators
A = @(x) real(ifft2(K.*fft2(x)));
AT = @(x) real(ifft2(KC.*fft2(x)));

% denoising function;
chambolleit = 5
Psi = @(x,th)  mex_vartotale(x,th,'itmax',chambolleit);
% TV regularizer;
Phi = @(x) TVnorm(x);


% start with the wiener filter
varx = var(y(:)); 	% approximate var of x
x0 = real(fft2(KC./(abs(KC).^2+10*sigma^2/varx).*ifft2(y)));

tolA = 1e-4;
% -- TwIST ---------------------------
% stop criterium:  the relative change in the objective function 
% falls below 'ToleranceA'
[x_twist,dummy,obj_twist,...
    times_twist,dummy,mse_twist]= ...
         TwIST(y,A,tau,...
         'AT', AT, ...
         'True_x', x,...       
         'Psi', Psi, ...
         'Phi',Phi, ...
         'Initialization',x0,...
         'Monotone',1,...
         'StopCriterion',1,...
       	 'ToleranceA',tolA,...
         'Verbose', 1);
     
         
figure(3); colormap gray; 
title('TwIST restored image')
imagesc(x_twist); axis off;
drawnow;


     
   
figure(5)
subplot(2,1,1)
plot(times_twist,obj_twist,'r','LineWidth',2,'LineWidth',2)
legend('TwIST')
st=sprintf('tau = %2.2e, sigma = %2.2e',tau,sigma),...
title(st)
ylabel('Obj. function')
xlabel('CPU time (sec)')
grid on

subplot(2,1,2)
plot(times_twist, (mse_twist*prod(size(x))).^0.5/norm(x,'fro'),'LineWidth',2);
title('error ||x^{t}-x||_2/||x||_2')
xlabel('CPU time (sec)')
grid on


fprintf(1,'TwIST   CPU time - %f\n', times_twist(end));


